function cma(x::AbstractVector{T}, L=30, mu=1e-4) where {T <: Complex}
    # initialize coefficients
    wk = zeros(T, L)
    wk[cld(L, 2) |> Int] = 1

    K = (length(x) - L + 1) ÷ 2
    # Allocate output vectors
    y = fill(T(NaN), K)
    e = fill(real(T)(NaN), K)
    # Load constellation
    C = [-1-1im, +1-1im, -1+1im, +1+1im]
    # Calculate radius
    R = sum(x -> abs2(x)^2, C) / sum(abs2, C)
    
    for k ∈ range(1, K)
        # Get k-th input vector
        @views xk = x[range(2(k-1) + 1, 2(k-1) + L)]
        # Apply equalizer
        y[k] = transpose(xk) * wk
        # Compute CMA error
        e[k] = R - abs2(y[k])
        # Update coefficients
        @. wk += 2mu * e[k] * y[k] * conj(xk)
    end

    return y, e, wk
end

function main()
    N = 10_000
    x = randn(ComplexF64, N)

    nruns = 100
    stats = @timed for k ∈ range(1, nruns)
        cma(x)
    end

    display("Runtime (ms): " * string(stats.time / nruns * 1e3))
end

main()