import numpy as np
import timeit

def cma(x, L=30, mu=1e-4):
    # load constellation
    C = np.array([-1-1j, +1-1j, -1+1j, +1+1j])
    # Initialize coefficients
    wk = np.zeros(L, dtype=complex)
    wk[L // 2] = 1
    # length of output sequence
    K = (np.size(x,0) - L + 1) // 2
    # Allocate output vectors
    y = np.zeros(K, dtype=complex)
    e = np.zeros(K)
    R = np.sum(np.power(np.abs(C), 4)) / np.sum(np.square(np.abs(C)))

    for k in range(0,K):
        # Get k-th input vector
        xk = x[range(2*k, 2*k + L)]
        # Apply equalizer
        y[k] = np.dot(np.transpose(xk), wk)
        # Compute CMA error
        e[k] = R - np.square(np.abs(y[k]))
        # Update coefficients
        wk += 2*mu*e[k]*y[k]*np.conj(xk)
    return y, e, wk


if __name__ == "__main__":
    N = 10_000
    x = np.random.randn(N) + 1j*np.random.randn(N)

    nruns = 100
    print("Runtime (ms): " + str(timeit.Timer(lambda: cma(x)).timeit(number=nruns) / nruns * 1e3))
