import numpy as np
import timeit

def ft_phase_screen(N, r0=0.1, seed=0, Nsh=0):
    rng = np.random.default_rng(seed)
    s = np.squeeze(rng.standard_normal(size=(N, N, 2)).view(complex))
    
    # side length of phase screen
    D = 1
    # outer scale of turbulence
    L0 = 1e2
    
    # angular spatial frequency vectors
    dκ = 2*np.pi/D
    κ = np.fft.fftfreq(N, 1/N) * dκ
    κx, κy = np.meshgrid(κ, κ)
    κ2 = κx ** 2 + κy ** 2

    # assume von Karman spectrum
    c = np.sqrt(0.49 * (r0) ** (-5/3)) * dκ
    s *= c * (κ2 + (2*np.pi/L0) ** 2) ** (-11/12)
    s[0,0] = 0

    screen = np.real(np.fft.ifft2(s, norm="forward"))

    if Nsh > 0:
        dx = D/N
        x = np.fft.fftshift(np.fft.fftfreq(N, 1/N) * dx)
        xx, xy = np.meshgrid(x, x)
        
        for p in range(1, Nsh+1):
            # setup the PSD
            dκ = 2*np.pi / D / 3**p
            κsh = np.fft.fftfreq(3, 1/3 / dκ)
            # frequency grid [rad/m]
            κx, κy = np.meshgrid(κsh, κsh)
            κ2 = κx ** 2 + κy ** 2

            # random draws of complex Fourier coefficients, twice the variance so that both created phase screens contain the full amount of power
            s = np.squeeze(rng.standard_normal(size=(3, 3, 2)).view(complex)) 
            c = np.sqrt(0.49 * r0**(-5/3)) * dκ
            s *= c * (κ2 + (2*np.pi/L0) ** 2) ** (-11/12)
            
            # loop over frequencies on subharmonic grid
            for (sk, κxk, κyk) in zip(s.flat, κx.flat, κy.flat):
                screen += np.real(sk * np.exp(1j*(κxk * xx + κyk * xy)))
    
    return screen

def main(N=256, Nsh=0):
    nruns = 100
    T = timeit.Timer(lambda: ft_phase_screen(N, Nsh=Nsh)).timeit(nruns) / nruns
    print("Runtime of pure-python sum (ms): " + str(T * 1e3))

if __name__ == "__main__":
    main()